# loading data
data <- read.xlsx("database.xlsx",sheet="DataFreezed",detectDates=T, na.strings="NA")

# calculating 2014-2023 averages and heating days
data <- data%>%
  mutate(gr = cut(yday(Dates),breaks=c(seq(0,360,7),366),labels=seq(0,360,7)))%>%
  group_by(gr)%>%
  mutate(
    IPLm=mean(IPL, na.rm=T),
    IDEm=mean(IDE, na.rm=T),
    ICZm=mean(ICZ, na.rm=T),
    INLm=mean(INL, na.rm=T),
    IITm=mean(IIT, na.rm=T),
    IATm=mean(IAT, na.rm=T),
    IESm=mean(IES, na.rm=T),
    TPLh = (15.5-TPL)*(TPL<15.5),
    TDEh = (15.5-TDE)*(TDE<15.5),
    TCZh = (15.5-TCZ)*(TCZ<15.5),
    TNLh = (15.5-TNL)*(TNL<15.5),
    TITh = (15.5-TIT)*(TIT<15.5),
    TATh = (15.5-TAT)*(TAT<15.5),
    TESh = (15.-TES)*(TES<15.5)
  )%>%
  ungroup() %>%
  select(!"gr")

# selecting observations with BRENT and EUR observations and filling missings
dt0   <- data %>%
  filter(!is.na(BRENT) & !is.na(EUR)) %>% 
  na.locf(maxgap = 6,na.rm=F) %>%
  na.omit()

# detects missing observations
m0           <- do.call(rbind,lapply(1:dim(dt0)[2],function(n){sum(is.na(dt0[,n]))}))
rownames(m0) <- colnames(dt0)
m0

# All prices in EUR/MWh
dt <- dt0%>%
  mutate(
    PPL      = PPL/EUR,
    BRENT    = (BRENT/EURUSD)/1.7,
    API      = (API/EURUSD)/6.15
    )

# Adding first differences
dt <- dt%>%
  mutate(
    dPPL     = c(NA,diff(log(PPL))),
    dPDE     = c(NA,diff(log(PDE))),
    dPNL     = c(NA,diff(log(PNL))),
    dPAT     = c(NA,diff(log(PAT))),
    dPCZ     = c(NA,diff(log(PCZ))),
    dPIT     = c(NA,diff(log(PIT))),
    dPES     = c(NA,diff(log(PES))),
    dEUA     = c(NA,diff(log(EUA))),
    dBRENT   = c(NA,diff(log(BRENT))),
    dAPI     = c(NA,diff(log(API)))
    )

# Adding ratios and differences relative to TTF
dt <- dt%>%
  mutate(
    PPLr     = log(PPL/PNL),
    PDEr     = log(PDE/PNL),
    PATr     = log(PAT/PNL),
    PCZr     = log(PCZ/PNL),
    PITr     = log(PIT/PNL),
    PESr     = log(PES/PNL),
    APIr     = log(API/PNL),
    BRENTr   = log(BRENT/PNL)
  ) %>%  
  mutate(
    PPLd     = PPL-PNL,
    PDEd     = PDE-PNL,
    PATd     = PAT-PNL,
    PCZd     = PCZ-PNL,
    PITd     = PIT-PNL,
    PESd     = PES-PNL,
    APId     = API-PNL,
    BRENTd   = BRENT-PNL
  ) %>%  
  mutate(
    IPLd     = IPL-IPLm,
    IDEd     = IDE-IDEm,
    IATd     = IAT-IATm,
    ICZd     = ICZ-ICZm,
    IITd     = IIT-IITm,
    IESd     = IES-IESm,
    INLd     = INL-INLm
  ) %>%  
  mutate(
    IPLdm     = IPLd-INLd,
    IDEdm     = IDEd-INLd,
    IATdm     = IATd-INLd,
    ICZdm     = ICZd-INLd,
    IITdm     = IITd-INLd,
    IESdm     = IESd-INLd
  )

str(dt)

# setting the sample
startD <- as.Date("2018-01-01")
dtFull = dt
dt <- dt[dt$Dates>=startD,]

# zoo object
#yts  <- zoo(dt[,-1],order.by=as.Date(dt[,1],"%Y-%m-%d"))
yts  <- zoo(dt[,-1],order.by=dt$Dates)
yts  <- na.omit(yts)


save(yts,dt,file="Data.Rdata")


# Plots and tables
#########################
PLOTSAVE=FALSE

# Inventory

if(PLOTSAVE==TRUE){cairo_pdf("FIG_Inventory.pdf",width=8,height=8,pointsize=10)}

par(mfrow=c(3,2),mar=c(2,4,2,2),bty="n", cex=0.7)
plot(yts[,c("IDE","IDEm")], screens=c(1,1), col=1:2, lty=1:2, ylim=c(10,100), ylab="", xlab="", main="Germany")
grid(nx=NULL,ny=NULL)
plot(yts[,c("IPL","IPLm")], screens=c(1,1), col=1:2, lty=1:2, ylim=c(10,100), ylab="", xlab="", main="Poland")
grid(nx=NULL,ny=NULL)
plot(yts[,c("IAT","IATm")], screens=c(1,1), col=1:2, lty=1:2, ylim=c(10,100), ylab="", xlab="", main="Austria")
grid(nx=NULL,ny=NULL)
plot(yts[,c("ICZ","ICZm")], screens=c(1,1), col=1:2, lty=1:2, ylim=c(10,100), ylab="", xlab="", main="Czechia")
grid(nx=NULL,ny=NULL)
plot(yts[,c("IIT","IITm")], screens=c(1,1), col=1:2, lty=1:2, ylim=c(10,100), ylab="", xlab="", main="Italy")
grid(nx=NULL,ny=NULL)
plot(yts[,c("IES","IESm")], screens=c(1,1), col=1:2, lty=1:2, ylim=c(10,100), ylab="", xlab="", main="Spain")
grid(nx=NULL,ny=NULL)

if(PLOTSAVE==TRUE){dev.off()}

# Temperature
if(PLOTSAVE==TRUE){cairo_pdf("FIG_Temperature.pdf",width=8,height=8,pointsize=10)}

par(mfrow=c(3,2),mar=c(2,4,2,2),bty="n", cex=0.7)
plot(yts$TDEh, main = "Germany", ylim = c(0,40), ylab="", xlab="")
abline(h=0, col=2);grid(nx=NULL,ny=NULL)
plot(yts$TPLh, main = "Poland", ylim = c(0,40), ylab="", xlab="")
abline(h=0, col=2);grid(nx=NULL,ny=NULL)
plot(yts$TATh, main = "Austria", ylim = c(0,40), ylab="", xlab="")
abline(h=0, col=2);grid(nx=NULL,ny=NULL)
plot(yts$TCZh, main = "Czechia", ylim = c(0,40), ylab="", xlab="")
abline(h=0, col=2);grid(nx=NULL,ny=NULL)
plot(yts$TITh, main = "Italy", ylim = c(0,40), ylab="", xlab="")
abline(h=0, col=2);grid(nx=NULL,ny=NULL)
plot(yts$TESh, main = "Spain", ylim = c(0,40), ylab="", xlab="")
abline(h=0, col=2);grid(nx=NULL,ny=NULL)

if(PLOTSAVE==TRUE){dev.off()}

# TTF and other commodities

if(PLOTSAVE==TRUE){cairo_pdf("FIG_EnCom.pdf",width=8,height=8,pointsize=10)}

par(mfrow=c(1,1),mar=c(2,4,2,2),bty="n", cex=1)
plot(yts[,c("PNL","BRENT","API")], screens=c(1,1,1), col=c(4,2,1), lwd=c(2,2,2), lty=c(1,1,1), ylim=c(0,300), ylab="EUR/MWh", xlab="", main=NULL)
grid(nx=NULL,ny=NULL)
legend(as.Date("2018-01-01"), 250, c("natural gas", "crude oil", "coal"), col=c(4,2,1), lwd=c(2,2,2), lty=c(1,1,1), bty = "n")
#plot(exp(yts[,c("BRENTr","APIr")]), screens=c(1,1), col=c(2,1), lwd=c(2,2), lty=c(1,1), ylim=c(0,6.2), ylab="Natural gas price = 1", xlab="", main="Ratio")
#abline(h=1); grid(nx=NULL,ny=NULL)
#legend(as.Date("2021-12-01"), 5, c("crude oil", "coal"), col=c(2,1), lwd=c(2,2), lty=c(1,1), bty = "n")

if(PLOTSAVE==TRUE){dev.off()}

# Price disparity -- differences
if(PLOTSAVE==TRUE){cairo_pdf("FIG_Diffs.pdf",width=8,height=10,pointsize=10)}

par(mfrow=c(4,2),mar=c(2,4,2,2),bty="n", cex=0.7)
plot(yts$PDEd, main = "Germany", ylim = c(-50,30), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(yts$PPLd, main = "Poland", ylim = c(-50,30), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(yts$PATd, main = "Austria", ylim = c(-50,30), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(yts$PCZd, main = "Czechia", ylim = c(-50,30), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(yts$PITd, main = "Italy", ylim = c(-50,30), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(yts$PESd, main = "Spain", ylim = c(-150,30), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(yts$BRENTd, main = "crude oil", ylim = c(-275,50), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(yts$APId, main = "coal", ylim = c(-275,50), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)

if(PLOTSAVE==TRUE){dev.off()}

# Price gwoth rates

if(PLOTSAVE==TRUE){cairo_pdf("FIG_d.pdf",width=8,height=8,pointsize=10)}

par(mfrow=c(3,3),mar=c(2,4,2,2),bty="n", cex=0.7)
plot(100*yts$dPNL, main = "Netherlands", ylim = c(-100,100), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dPDE, main = "Germany", ylim = c(-100,100), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dPPL, main = "Poland", ylim = c(-100,100), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dPAT, main = "Austria", ylim = c(-100,100), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dPCZ, main = "Czechia", ylim = c(-100,100), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dPIT, main = "Italy", ylim = c(-100,100), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dPES, main = "Spain", ylim = c(-50,50), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dBRENT, main = "Crude oil", ylim = c(-50,50), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)
plot(100*yts$dAPI, main = "Coal", ylim = c(-50,50), ylab="", xlab="")
abline(h=1);grid(nx=NULL,ny=NULL)

if(PLOTSAVE==TRUE){dev.off()}

# Price disparity -- ratios

if(PLOTSAVE==TRUE){cairo_pdf("FIG_Ratios.pdf",width=8,height=10,pointsize=10)}

par(mfrow=c(4,2),mar=c(2,4,2,2),bty="n", cex=0.7)
plot(100*(yts$PDEr), main = "Germany", ylim = c(-100,100), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)
plot(100*(yts$PPLr), main = "Poland", ylim = c(-100,100), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)
plot(100*(yts$PATr), main = "Austria", ylim = c(-100,100), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)
plot(100*(yts$PCZr), main = "Czechia", ylim = c(-100,100), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)
plot(100*(yts$PITr), main = "Italy", ylim = c(-100,100), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)
plot(100*(yts$PESr), main = "Spain", ylim = c(-100,100), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)
plot(100*(yts$BRENTr), main = "crude oil", ylim = c(-200,200), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)
plot(100*(yts$APIr), main = "coal", ylim = c(-200,200), ylab="", xlab="")
abline(h=0);grid(nx=NULL,ny=NULL)


if(PLOTSAVE==TRUE){dev.off()}


# Descriptiove statistics
WB <- createWorkbook("NG")


# Correlations
options(digits=3)
tmp <- yts[,c("dPNL","dPDE","dPPL","dPAT","dPCZ","dPIT","dPES","dBRENT","dAPI")]
names(tmp) <- c("NG NL","NG DE","NG PL","NG AT","NG CZ","NG IT","NG ES","OIL","COAL")
tmp1 = cor(tmp)
tmp1
addWorksheet(WB,"Correlations")
writeData(WB,sheet="Correlations",x=tmp1 ,startCol=2,startRow=2,rowNames=T)

#require(corrplot)

# Colors are here https://cran.r-project.org/web/packages/corrplot/vignettes/corrplot-intro.html

if(PLOTSAVE==TRUE){cairo_pdf("FIG_Cor.pdf",width=8,height=8,pointsize=10)}

corrplot.mixed(tmp1,
               upper.col = COL1('Greys',n=20),
               col.lim = c(0,1),
               number.cex  = 0.8,
               lower = "number",
               tl.col = "black",
               tl.cex = 0.8,
               tl.pos = "lt",
               lower.col = "black") 



if(PLOTSAVE==TRUE){dev.off()}

# Descriptive stats
tmp1 <- na.omit(yts[,c("dPNL","dPDE","dPPL","dPAT","dPCZ","dPIT","dPES","dEUA","dBRENT","dAPI")]*100)

# growth ratets
statsD<-
  round(cbind(
    apply(tmp1,2,mean)*259,
    apply(tmp1,2,sd)*sqrt(259),
    apply(tmp1,2,min),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.25))),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.50))),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.75))),
    apply(tmp1,2,max),
    apply(tmp1,2,skewness),
    apply(tmp1,2,kurtosis),
    apply(tmp1,2,function(x) ur.df(x,type="drift",lags=1,selectlags="Fixed")@teststat[1]),
    apply(tmp1,2,function(x) ur.df(x,type="drift",lags=1,selectlags="Fixed")@cval[1]),
    apply(tmp1,2,function(x) Acf(as.numeric(x),plot = FALSE)$acf[2]),
    apply(tmp1,2,function(x) Box.test(x  ,lag=1,type="Ljung-Box")$p.value),
    apply(tmp1,2,function(x) Acf(as.numeric(x^2),plot = FALSE)$acf[2]),
    apply(tmp1,2,function(x) Box.test(x^2,lag=1,type="Ljung-Box")$p.value)),5)
colnames(statsD) <- c("Mean","SD","Min.","P25","P50","P75","Max.","Skew.","Kurt.","ADF","ADF 1%","$\\rho$","$\\rho$ p-val","ARCH","ARCH p-val")

addWorksheet(WB,"statsD")
writeData(WB,sheet="statsD",x=statsD ,startCol=2,startRow=2,rowNames=T)

# differences
tmp1 <- na.omit(yts[,c("PDEd","PPLd","PATd","PCZd","PITd","PESd","BRENTd","APId")])
statsDif<-
  round(cbind(
    apply(tmp1,2,mean),
    apply(tmp1,2,sd),
    apply(tmp1,2,min),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.25))),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.50))),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.75))),
    apply(tmp1,2,max),
    apply(tmp1,2,skewness),
    apply(tmp1,2,kurtosis),
    apply(tmp1,2,function(x) ur.df(x,type="drift",lags=1,selectlags="Fixed")@teststat[1]),
    apply(tmp1,2,function(x) ur.df(x,type="drift",lags=1,selectlags="Fixed")@cval[1]),
    apply(tmp1,2,function(x) Acf(as.numeric(x),plot = FALSE)$acf[2]),
    apply(tmp1,2,function(x) Box.test(x  ,lag=1,type="Ljung-Box")$p.value),
    apply(tmp1,2,function(x) Acf(as.numeric(x^2),plot = FALSE)$acf[2]),
    apply(tmp1,2,function(x) Box.test(x^2,lag=1,type="Ljung-Box")$p.value)),5)
colnames(statsDif) <- c("Mean","SD","Min.","P25","P50","P75","Max.","Skew.","Kurt.","ADF","ADF 1%","$\\rho$","$\\rho$ p-val","ARCH","ARCH p-val")

addWorksheet(WB,"statsDif")
writeData(WB,sheet="statsDif",x=statsDif ,startCol=2,startRow=2,rowNames=T)


# ratios
tmp1 <- na.omit(yts[,c("PDEr","PPLr","PATr","PCZr","PITr","PESr","BRENTr","APIr")])*100
statsR<-
  round(cbind(
    apply(tmp1,2,mean),
    apply(tmp1,2,sd),
    apply(tmp1,2,min),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.25))),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.50))),
    apply(tmp1,2,function(x) quantile(x,probs=c(0.75))),
    apply(tmp1,2,max),
    apply(tmp1,2,skewness),
    apply(tmp1,2,kurtosis),
    apply(tmp1,2,function(x) ur.df(x,type="drift",lags=1,selectlags="Fixed")@teststat[1]),
    apply(tmp1,2,function(x) ur.df(x,type="drift",lags=1,selectlags="Fixed")@cval[1]),
    apply(tmp1,2,function(x) Acf(as.numeric(x),plot = FALSE)$acf[2]),
    apply(tmp1,2,function(x) Box.test(x  ,lag=1,type="Ljung-Box")$p.value),
    apply(tmp1,2,function(x) Acf(as.numeric(x^2),plot = FALSE)$acf[2]),
    apply(tmp1,2,function(x) Box.test(x^2,lag=1,type="Ljung-Box")$p.value)),5)
colnames(statsR) <- c("Mean","SD","Min.","P25","P50","P75","Max.","Skew.","Kurt.","ADF","ADF 1%","$\\rho$","$\\rho$ p-val","ARCH","ARCH p-val")

addWorksheet(WB,"statsR")
writeData(WB,sheet="statsR",x=statsR ,startCol=2,startRow=2,rowNames=T)


saveWorkbook(WB,"outcomes.xlsx",overwrite=T)

